<?php
/**
 * 2年間利用がなかったユーザの自動削除
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2021, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 *
 * 実行例）
 * http://localhost/map/batch_delete_accounts/batch
 * https://map.gakunin.nii.ac.jp/map/batch_delete_accounts/batch
 * https://test-map.gakunin.nii.ac.jp/map/batch_delete_accounts/batch
 * テスト用）
 * http://localhost/map/batch_delete_accounts/batch?dryrun=1
 */
class BatchDeleteAccountsController extends AppController {
	var $name = 'BatchSendNoticeMails';
	var $uses = array('AccountEppn', 'Group', 'SentNoticeMailLog', 'Mygroup', 'Transaction',
		'GroupSphost', 'AutoGroup');
	var $components = array('NoticeUtils');

	function beforeFilter() {
		// サーバ自身からのアクセスのみを許可する
		if ($_SERVER['REMOTE_ADDR'] != $_SERVER['SERVER_ADDR']) {
			$this->cakeError('cannotAccess', array());
			return;
		}
		$this->autoRender = false;
		$this->accessLog();
	}

	/* 日々のバッチ処理 */
	function batch() {
		if ($this->params['url']['url'] != 'batch_delete_accounts/batch') {
			return;
		}

		$this->log('BatchDeleteAccountsController start', "batch");

		$dryrun = 0;	// テスト用フラグ
		if (isset($this->params['url']['dryrun'])) {
			$this->log('BatchDeleteAccountsController * For testing purposes, no DB changes will be performed. *', "batch");
			$dryrun = 1;
		}

		$startTime = date('Y-m-d H:i:s');

		// キャッシュを無効化
		$this->AccountEppn->cacheQueries = false;
		$this->Group->cacheQueries = false;
		$this->SentNoticeMailLog->cacheQueries = false;
		$this->Mygroup->cacheQueries = false;
		$this->GroupSphost->cacheQueries = false;
		$this->AutoGroup->cacheQueries = false;

		// 削除対象アカウントの洗い出し
		$accountEppns = $this->AccountEppn->find('all', array(
			'fields' => array('id', 'account_id', 'eppn', 'entityid'),
			'conditions' => array(
				'last_login_approx <= (? - INTERVAL 24 MONTH)' => array($startTime),
				'OR' => array(
                                	'id in (select account_eppn_id from sent_notice_mail_logs where kind = 2 and pending is null)',
                                	"account_id in (select id from accounts where mail = '' or mail is null)"
				)
			),
			'order' => array('id DESC')
		));
		if (empty($accountEppns)) {
			$this->log('BatchDeleteAccountsController : No account to be delete', "batch");
			$this->log('BatchDeleteAccountsController end', "batch");
			return;
		}
		$this->log('BatchDeleteAccountsController : Number of accounts to be deleted :' . count($accountEppns), "batch");
		// 削除対象になっているアカウントのリスト
		$deleteIdList = $this->SentNoticeMailLog->find('list', array(
			'fields' => array('account_eppn_id'),
		));
		$deleteIdList = array_unique($deleteIdList);
		$eppnList = array();
		$pendingList = array();
		foreach ($accountEppns as $key => &$aEppn) {
			$eppnId = $aEppn['AccountEppn']['id'];
			$accountId = $aEppn['AccountEppn']['account_id'];
			$eppn = $aEppn['AccountEppn']['eppn'];
			$delete = true;
			$count = $this->AccountEppn->find('count', array('conditions' => array('account_id' => $accountId)));
			if ($count > 1) {	// 複数のePPNと紐付けられている
				$delete = false;
			}
			if ($delete) {
				// 単独管理者グループ洗い出し
				$soloAdminGroups = $this->NoticeUtils->getSoloAdminGroups($accountId, $deleteIdList);
				if (!empty($soloAdminGroups)) {	// アカウント削除で単独管理者グループあり
					$this->log("BatchDeleteAccountsController : pending Account(id = $accountId) with $eppn(id = $eppnId)", "batch");
					if (!$dryrun) {
						$this->SentNoticeMailLog->updateAll(array('pending' => 1),
							array('account_eppn_id' => $eppnId));
					}
					$pending['eppn'] = $eppn;
					$pending['group'] = implode(", ", $this->Group->findGroupNameList($soloAdminGroups));
					$pendingList[] = $pending;
					continue;
				}
			}
			// アカウント削除
			if (!$dryrun) {
				$this->Transaction->begin();
				if (!$delete) {	// リンク解除
					$this->log("BatchDeleteAccountsController : unlink Account(id = $accountId) and delete $eppn(id = $eppnId)", "batch");
					// 機関グループに参加している場合は、退会処理をする
					$idpEntityid = $aEppn['AccountEppn']['entityid'];
					$count = $this->AccountEppn->find('count', array('conditions' => array(
						'account_id' => $accountId, 'entityid' => $idpEntityid)));
					if ($count == 1) {	// 他に同じentityidが無い
						$autoGroup = $this->AutoGroup->findIdPGroup($idpEntityid);
						if (!empty($autoGroup)) {
							$idpGroupId = $autoGroup['AutoGroup']['group_id'];
							// プライベートサービスの管理者情報を削除
							$privateServiceIdList = $this->GroupSphost->getPrivateServiceId($idpGroupId);
							$this->Mygroup->deleteAll(array('groupid' => $privateServiceIdList,
								'account_id' => $accountId), false);
							// Myグループ情報を削除
							$this->Mygroup->deleteAll(array('groupid' => $idpGroupId,
								'account_id' => $accountId), false);
						}
					}
					$this->AccountEppn->delete($eppnId);
					$this->SentNoticeMailLog->deleteAll(array('account_eppn_id' => $eppnId), false);
				}
				else {	// アカウント削除
					$this->log("BatchDeleteAccountsController : delete Account(id = $accountId) with $eppn(id = $eppnId).", "batch");
					if (!$this->NoticeUtils->deleteAccountGroup($accountId)) {
						$this->Transaction->rollback();
						$this->log("BatchDeleteAccountsController ERROR delete Account(id = $accountId) delete Error", "batch");
						break;
					}
				}
				$this->Transaction->commit();
			}
			$eppnList[] = $eppn;
		}
		$this->set('eppnList', $eppnList);
		$this->debugLog('BatchDeleteAccountsController::batch $eppnList => '.var_export($eppnList, true));
		$this->set('pendingList', $pendingList);
		$this->debugLog('BatchDeleteAccountsController::batch $pendingList => '.var_export($pendingList, true));

		if (!$dryrun && (!empty($eppnList) || !empty($pendingList))) {
			$this->sendMail('delete.eppns', Configure::read('system_admin_email'), 'ja',
				'delete_accounts_template');
		}

		$this->log('BatchDeleteAccountsController end', "batch");
	}
}
?>
