<?php
/**
 * AccountEppn Model
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class AccountEppn extends AppModel{
	var $name = 'AccountEppn';
	var $useTable = 'account_eppns';
	var $validate = array(
		'eppn' => array(
			'rule' => array('requireEppn')
	),
	);

	/**
	 * eppn重複チェック
	 */
	function requireEppn($values) {
		$eppn = isset($values['eppn']) ? $values['eppn'] : '';
		$eptid = isset($this->data['AccountEppn']['eptid']) ? $this->data['AccountEppn']['eptid'] : '';
		if ($eppn == 'guest' && $eptid == 'guest') {
			return true;	// ゲストユーザ
		}

		if (empty($this->data['AccountEppn']['id'])) { // 新規作成の場合
			$id = -1;
		}
		else {
			$id = $this->data['AccountEppn']['id'];
		}

		$count = $this->find('count', array(
			'conditions' => array(
				'eppn' => $eppn, 'NOT' => array('id' => $id))));
		if ($count >= 1) {
			$this->log('AccountEppn#requireEppn eppn('.$eppn.') is already registered.');
			return false;
		}
		return true;
	}

	function findByAccountId($userid) {
		return $this->find('all', array('conditions' => array('account_id' => $userid)));
	}

	/**
	 * AccountEppnを生成します
	 */
	public function createAccountEppn($accountId, $eppn, $eptid, $entityid) {
		if (empty($eppn)) {
			$this->log('AccountEppn#createAccountEppn NO ePPN Error');
			return false;
		}

		// 同じeppnが存在しない場合のみINSERTする
		if (empty($eptid)) {
			$eptid = NULL;
		}

		$sql = "INSERT INTO account_eppns (account_id, eppn, eptid, entityid, modified, created) ";
		$sql .= "SELECT ?, ?, ?, ?, ?, ? FROM dual ";
		$sql .= "WHERE NOT EXISTS (SELECT eppn FROM account_eppns WHERE eppn = ? FOR UPDATE) LIMIT 1;";

		$result = false;
		$count = 0;
		$max = 5; // 最大試行回数
		$maximumBackoff = 32000; // リトライ間隔の最大値（ミリ秒）
		do {
			$now = date('Y-m-d H:i:s');
			$result = $this->query($sql, array($accountId, $eppn, $eptid, $entityid, $now, $now, $eppn), false);
			if (!$result) {
				$error = $this->getDataSource()->error;
				$err = explode(':', $error);
				if ($err[0] != 1213) { // Deadlock以外のエラーは終了
					$this->log('AccountEppn#createAccountEppn INSERT Error '.$error);
					return false;
				}
				if ($count >= $max) {
					$this->log('AccountEppn#createAccountEppn INSERT retry max Error ');
					return false;
				}
				// ランダムに0から1秒を加えたExponential Backoff(ミリ秒）
				$randSleepTime = min(((pow(2,$count) * 1000) + rand(0,1000)), $maximumBackoff);
				usleep($randSleepTime * 1000);
			}
			$count++;
		} while (!$result);
		$count = $this->query('SELECT ROW_COUNT() AS rowCount');
		if (!isset($count[0][0]['rowCount'])) {
			return false;
		}
		if ($count[0][0]['rowCount'] == 0) { // INSERTされなかった
			$this->log('AccountEppn#createAccountEppn eppn('.$eppn.') is already registered.');
			return false;
		}
	
		return true;
	}

	/**
	 * ゲストユーザを検索します。
	 * @param $entityid IdPのエンティティID
	 */
	public function findGuest($entityid) {
		$accountEppn = $this->find('first', array('conditions' => array(
				'eppn' => 'guest',
				'eptid' => 'guest',
				'entityid' => $entityid)));
		return $accountEppn;
	}

	/**
	 * ゲスト用AccountEppnを生成します
	 */
	public function createGuestAccountEppn($accountId, $entityid) {
		$accountEppn = array(
				'account_id' => $accountId,
				'entityid' => $entityid,
				'eppn' => 'guest',
				'eptid' => 'guest',
		);

		$this->create(false);	// デフォルト値を設定しない
		$result = $this->save($accountEppn, false);
		if (!$result) {
			return false;
		}
		return true;
	}

	/**
	 * ePPNに紐づくアカウントIDを検索します。
	 * @param $eppnList ePPN一覧
	 */
	public function findAccountIdListByEppn($eppnList) {
		$list = $this->find('list', array(
				'fields' => array('account_id', 'account_id'),
				'conditions' => array('eppn' => $eppnList)));
		if (empty($list)) {
			return false;
		}
		return $list;
	}

	/**
	 * 複数のePPNと紐付けられているアカウントのidとaccount_idを返します
	 */
	public function getLinkedAccounts() {
		$linkedAccounts = $this->find('all', array(
			'fields' => array('id', 'account_id'),
			'conditions' => array('exists (select * from account_eppns t2 where AccountEppn.account_id = t2.account_id group by t2.account_id having count(t2.account_id) > 1)'),
			'order' => array('account_id')
		));
		return $linkedAccounts;
	}

	/**
	 * 同一メールアドレスで別アカウントのidとaccount_idを返します
	 */
	public function getSameMails() {
		$sameMails = $this->find('all', array(
			'fields' => array('AccountEppn.id', 'AccountEppn.account_id', 'Account.mail'),
			'joins' =>array(array(
				'type'=>'INNER', 'table'=>'accounts', 'alias'=>'Account',
				'conditions' => array(
					'Account.id = AccountEppn.account_id',
				))
			),
			'conditions' => array('exists (select * from accounts t2 where t2.is_temp = 0 and t2.mail is not null and t2.mail != "" and Account.mail = t2.mail group by t2.mail having count(t2.mail) > 1)'),
		));
		return $sameMails;
	}
}
?>
